/*****************************************************************
 *                GetSize NSIS plugin v1.0                       *
 *                                                               *
 * 2005 Shengalts Aleksander aka Instructor (Shengalts@mail.ru)  *
 *****************************************************************/

#include <windows.h>
#include <stdlib.h>

/* Include string functions */
#define GetOptions
#include "StrFunc.h"

/* NSIS stack structure */
typedef struct _stack_t {
	struct _stack_t *next;
	char text[1024];
} stack_t;

stack_t **g_stacktop;

/* Defines */
typedef void (*SearchFilesCallback)(char *filename, char *path, char *name, __int64 size);
#define IDC_STATIC      1006
#define NSIS_MAX_STRLEN 1024

/* Global variables */
__int64 dirsize=0;
int filesum=0;
int dirsum=0;
HANDLE hSearchDetails=0;

/* NSIS variables */
enum
{
INST_0,         // $0
INST_1,         // $1
INST_2,         // $2
INST_3,         // $3
INST_4,         // $4
INST_5,         // $5
INST_6,         // $6
INST_7,         // $7
INST_8,         // $8
INST_9,         // $9
INST_R0,        // $R0
INST_R1,        // $R1
INST_R2,        // $R2
INST_R3,        // $R3
INST_R4,        // $R4
INST_R5,        // $R5
INST_R6,        // $R6
INST_R7,        // $R7
INST_R8,        // $R8
INST_R9,        // $R9
INST_CMDLINE,   // $CMDLINE
INST_INSTDIR,   // $INSTDIR
INST_OUTDIR,    // $OUTDIR
INST_EXEDIR,    // $EXEDIR
INST_LANG,      // $LANGUAGE
__INST_LAST
};

/* Funtions prototypes and macros */
BOOL SearchFiles(char *path, char *wildcard, BOOL subdir, SearchFilesCallback lpAddSize);
void AddSize(char *filename, char *path, char *name, __int64 size);
int popstring(char *str);
int getvarindex(char *var);
#define setvar(varnum, var, g_stringsize, g_variables) {\
	if (var != NULL && varnum >= 0 && varnum < __INST_LAST)\
		lstrcpy(g_variables + varnum*g_stringsize, var);}

/* NSIS functions code */
void __declspec(dllexport) GetSize(HWND hwndParent, int string_size, 
                                      char *variables, stack_t **stacktop)
{
  g_stacktop=stacktop;
  {
	char szPath[NSIS_MAX_STRLEN]="";
	char szOptions[NSIS_MAX_STRLEN]="";
	char szError[4]="";
	char szBuf[NSIS_MAX_STRLEN]="";
	char szWildcard[MAX_PATH]="";
	char *pPath;
	int nVar1, nVar2, nVar3;
	int nDivisor=1;
	BOOL bGotoSubdir=TRUE;

	popstring(szPath);
	popstring(szOptions);
	popstring(szError);
	nVar1=getvarindex(szError);
	popstring(szError);
	nVar2=getvarindex(szError);
	popstring(szError);
	nVar3=getvarindex(szError);

	pPath=szPath + lstrlen(szPath);
	while (*--pPath == '\\') *pPath='\0';

	CharUpper(szBuf);
	if (GetOptions(szWildcard, szOptions, "/M="))
		lstrcpy(szWildcard, "*.*");
	if (!GetOptions(szBuf, szOptions, "/G=") && (*szBuf == '0'))
		bGotoSubdir=FALSE;
	if (!GetOptions(szBuf, szOptions, "/S="))
		if (*szBuf == 'K') nDivisor=1024;
		else if (*szBuf == 'M') nDivisor=1048576;
		else if (*szBuf == 'G') nDivisor=1073741824;
	hSearchDetails=GetDlgItem(FindWindowEx(hwndParent, NULL, "#32770", NULL), IDC_STATIC);

	if (SearchFiles(szPath, szWildcard, bGotoSubdir, AddSize))
	{
		dirsize=dirsize / nDivisor;
		_i64toa(dirsize, szBuf, 10);
		setvar(nVar1, szBuf, string_size, variables);
		_itoa(filesum, szBuf, 10);
		setvar(nVar2, szBuf, string_size, variables);
		_itoa(--dirsum, szBuf, 10);
		setvar(nVar3, szBuf, string_size, variables);
	}
	else
	{
		setvar(nVar1, "", string_size, variables);
		setvar(nVar2, "", string_size, variables);
		setvar(nVar3, "", string_size, variables);
	}
	SendMessage(hSearchDetails, WM_SETTEXT, 0, (LPARAM)"");
  }
}

BOOL WINAPI DllMain(HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
{
	return TRUE;
}

BOOL SearchFiles(char *path, char *wildcard, BOOL subdir, SearchFilesCallback lpAddSize)
{
	char name[MAX_PATH];
	HANDLE hSearch=NULL;
	WIN32_FIND_DATA data;

	if (subdir)
	{
		lstrcpy(name, path);
		lstrcat(name, "\\");
		lstrcat(name, "*.*");

		if ((hSearch=FindFirstFile(name, &data)) != INVALID_HANDLE_VALUE)
			do
			{
				if (data.cFileName[0] == '.')
					continue;

				if (data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
				{
					lstrcpy(name, path);
					lstrcat(name, "\\");
					lstrcat(name, data.cFileName);

					SearchFiles(name, wildcard, TRUE, lpAddSize);
				}
			}
			while (FindNextFile(hSearch, &data));
		else
			return FALSE;

		FindClose (hSearch);
	}

	lstrcpy(name, path);
	lstrcat(name, "\\");
	lstrcat(name, wildcard);

		/* Needless outside plugin */
		SendMessage(hSearchDetails, WM_SETTEXT, 0, (LPARAM)name);
		++dirsum;
		/***************************/

	if ((hSearch=FindFirstFile(name, &data)) == INVALID_HANDLE_VALUE)
		return TRUE;
	do
		if (!(data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
		{
			lstrcpy(name, path);
			lstrcat(name, "\\");
			lstrcat(name, data.cFileName);

			lpAddSize(name, path, data.cFileName, (data.nFileSizeHigh * ((__int64)MAXDWORD+1)) + data.nFileSizeLow);
		}
			/* Needless outside plugin */
			else if (!subdir)
				++dirsum;
			/***************************/
	while (FindNextFile(hSearch, &data));

	FindClose (hSearch);

	return TRUE;
}

void AddSize(char *filename, char *path, char *name, __int64 size)
{
	++filesum;
	dirsize+=size;
}

//Function: Removes the element from the top of the NSIS stack and puts it in the buffer.
int popstring(char *str)
{
  stack_t *th;
  if (!g_stacktop || !*g_stacktop) return 1;
  th=(*g_stacktop);
  lstrcpy(str,th->text);
  *g_stacktop = th->next;
  GlobalFree((HGLOBAL)th);
  return 0;
}

// Function: Converts .r0-.r9 and .R0-.R9 to variable index
int getvarindex(char *var)
{
	int nVar=-1;

	if (lstrcmp(var, ".r0") == 0)
		nVar=INST_0;
	else if (lstrcmp(var, ".r1") == 0)
		nVar=INST_1;
	else if (lstrcmp(var, ".r2") == 0)
		nVar=INST_2;
	else if (lstrcmp(var, ".r3") == 0)
		nVar=INST_3;
	else if (lstrcmp(var, ".r4") == 0)
		nVar=INST_4;
	else if (lstrcmp(var, ".r5") == 0)
		nVar=INST_5;
	else if (lstrcmp(var, ".r6") == 0)
		nVar=INST_6;
	else if (lstrcmp(var, ".r7") == 0)
		nVar=INST_7;
	else if (lstrcmp(var, ".r8") == 0)
		nVar=INST_8;
	else if (lstrcmp(var, ".r9") == 0)
		nVar=INST_9;
	else if (lstrcmp(var, ".R0") == 0)
		nVar=INST_R0;
	else if (lstrcmp(var, ".R1") == 0)
		nVar=INST_R1;
	else if (lstrcmp(var, ".R2") == 0)
		nVar=INST_R2;
	else if (lstrcmp(var, ".R3") == 0)
		nVar=INST_R3;
	else if (lstrcmp(var, ".R4") == 0)
		nVar=INST_R4;
	else if (lstrcmp(var, ".R5") == 0)
		nVar=INST_R5;
	else if (lstrcmp(var, ".R6") == 0)
		nVar=INST_R6;
	else if (lstrcmp(var, ".R7") == 0)
		nVar=INST_R7;
	else if (lstrcmp(var, ".R8") == 0)
		nVar=INST_R8;
	else if (lstrcmp(var, ".R9") == 0)
		nVar=INST_R9;

	return nVar;
}
